# Install packages if necessary
install.packages("vegan")
install.packages("stats")
install.packages("gdata")
install.packages("miscTools")

# Load packages
library(vegan)
library(stats)
library(gdata)
library(miscTools)

# Load data and coerce to matrix form
# In this case, the data are a date-by-shark presence-absence matrix for Y1ID frequent sharks.
# These are sharks that were initially identified in the first year of the study and were seen on > 90 days overall (see Methods).
mat <- read.table("Y1_frequent_data_thresh30avg.txt", header = TRUE)
mat <- as.matrix(mat)

# Code for test.a function... these are the permutation tests developed by P.L. Legendre (see Legendre & Legendre 1998)
test.a <-
    function (mat, nperm=9999)
    ### Function to compute and test the statistic 'a' by permutation.
    ### 'a' is the co-occurrence of the species. It is computed among the species.
    ### mat: site-by-species (or dates-by-sharks in this case) matrix. 
    ###      Sites are rows, species are columns.
    ### Output list --
    ###   a : Matrix of co-occurrence statistics 'a'.
    ###   p.a : Matrix of p-values associated with the 'a' statistics, in matrix form.
    ###   p.a.dist : Matrix of p-values associated with the a statistics in 'dist' form,
    ###              ready to be used as input in the hclus() clustering function.
{
	A <- system.time({
    mat <- as.matrix(mat)
    date <- rownames(mat)
    sharkID <- colnames(mat)

# Transform the 'pa' or abundance data to presence-absence
# mat <- decostand(mat, "pa")
# Not needed in this case since the shark data is already presence-absence

    n <- nrow(mat)
    p <- ncol(mat)
    a <- t(mat) %*% mat
    
    # Permutation tests for a
    p.a = matrix(1, p, p, dimnames = list(sharkID, sharkID))
    for(iperm in 1:nperm) {
        perm.mat = mat[sample(n),1]
        for(j in 2:p) {
            vec <- mat[sample(n),j]
            perm.mat <- cbind(perm.mat, vec)
            }
        #
        a.perm <- t(perm.mat) %*% perm.mat
        for(j in 2:p) {
            for(jj in 1:(p-1)) {
                if(a.perm[j,jj] >= a[j,jj]) p.a[j,jj] <- p.a[j,jj] + 1
                }
            }
        }
    p.a <- p.a/(nperm+1)
    
    for(j in 1:(p-1)) {
        for(jj in (j+1):p) {
            p.a[j,jj] <- NA
            }
        }
    diag(p.a) <- NA

	})
	A[3] <- sprintf("%2f",A[3])
	cat("Computation time =",A[3]," sec",'\n')
        
    out <- list(a=a, p.a=p.a, p.a.dist=as.dist(p.a))
    out
}

# Run test.a function on mat and store as object m1
m1 <- test.a(mat)

# Show outputs
print(mat) # Shark-by-date matrix
print(m1$a) # Co-occurrence matrix (frequency of co-occurrence of each pair of sharks)
print(m1$p.a) # Half-matrix of p-values
print(m1$p.a.dist) # Half-matrix without the NAs

# Convert half-matrix of p-values into symmetrical matrix to feed into PCoA function
lower <- lowerTriangle(m1$p.a, diag = TRUE)
print(lower)

complete <- symMatrix(lower)
print(complete)

# Add row and column names back in
rownames(complete) <- c("1", "2", "3", "4", "6", "8", "11", "18", "28", "37", "42", "43", "50", "57", "60", "61", "66", "76", "95", "104")
colnames(complete) <- c("1", "2", "3", "4", "6", "8", "11", "18", "28", "37", "42", "43", "50", "57", "60", "61", "66", "76", "95", "104")
print(complete)

# Perform Holm correction on half-matrix of p-values
# This will be used to determine the threshold value in the original matrix of p-values below which tests of a remain significant after Holm correction. This determines which pairs of sharks will have lines drawn in the PCoA plot (i.e., the 'thresh' parameter)
p.a.holm <- p.adjust(as.vector(m1$p.a), method = "holm")
print(p.a.holm)
# Using S1-S37 as an example (entry 10 in p.a.holm), 0.0007 before correction became 0.0693 after correction, so the threshold p-value in the original matrix that would remain significant after Holm correction would be p = 0.0007*0.05/0.0693 = 0.0005

0.0007*0.05/0.0693 # 0.0005 # entry 10 in p.a.holm (S1-S37)
0.0004*0.05/0.0404 # 0.0005 # entry 20 in p.a.holm (S1-S104)





####### PCoA function
Diss.mat <- complete
trace.liens <- function(Diss.mat, thresh=0.0005, xlim=NULL, ylim=NULL)
#
# Plot a PCoA graph from a dissimilarity **matrix**.
# Add lines corresponding to values below a dissimilarity threshold.
#
# Diss.mat = dissimilarity matrix, e.g. res.test.a.mat.9999.2[1:29,1:29]
{
X = cmdscale(as.dist(Diss.mat))
quartz(height = 4.2, width = 4.2)
par(mai = c(1.0, 1.0, 1.0, 0.5))
plot(X, type="p", xlim=xlim, ylim=ylim, pch = 16, cex = 0.8, asp=1, xlab="PCoA axis 1", ylab="PCoA axis 2")
text(X, labels=rownames(X), pos=1, cex = 0.8)
# title(main = c("Y1ID Frequent Sharks"), family="serif")
n = nrow(Diss.mat)
for(j in 1:(n-1)) {
   for(jj in (j+1):n) {
      if(Diss.mat[j,jj] <= thresh) lines(c(X[j,1], X[jj,1]), c(X[j,2], X[jj,2]), lwd = 0.2, col = "blue")
      }
   }
}

# Run PCoA function and show plot
trace.liens(Diss.mat)

# Check eigenvalues to validate model
test <- cmdscale(as.dist(Diss.mat), eig = TRUE)
test$eig
# Note the large number of negative eigenvalues, which indicate non-Euclideanarity in the matrix. To get around this, use the option
# add = TRUE in the cmdscale command. This adds a constant to all distances in the dissimilarity matrix (below). 





####### PCoA function
# Re-run the PCoA but specify that a constant must be added (Cailliez 1983), cited in Legendre & Legendre 1998 p. 434 (correction method # 2)
# In the R help file for cmdscale, it says this is what the add option does, citing Cailliez 1983
final <- cmdscale(as.dist(Diss.mat), add = TRUE, eig = TRUE)

Diss.mat <- complete
trace.liens <- function(Diss.mat, thresh=0.0005, xlim=NULL, ylim=NULL)
  #
  # Plot a PCoA graph from a dissimilarity **matrix**.
  # Add lines corresponding to values below a dissimilarity threshold.
  #
  # Diss.mat = dissimilarity matrix, e.g. res.test.a.mat.9999.2[1:29,1:29]
{
  X = final$points # cmdscale output becomes a list when the add and eig options are TRUE, so this line specifies the points object from that list as the input for the PCoA plot
  quartz(height = 4.2, width = 4.2)
  par(mai = c(1.0, 1.0, 1.0, 0.5))
  plot(X, type="p", xlim=xlim, ylim=ylim, pch = 16, cex = 0.8, asp=1, xlab="PCoA axis 1", ylab="PCoA axis 2")
  text(X, labels=rownames(X), pos=1, cex = 0.8)
  # title(main = c("Y1ID Frequent Sharks"), family="serif")
  n = nrow(Diss.mat)
  for(j in 1:(n-1)) {
    for(jj in (j+1):n) {
      if(Diss.mat[j,jj] <= thresh) lines(c(X[j,1], X[jj,1]), c(X[j,2], X[jj,2]), lwd = 0.2, col = "blue")
    }
  }
}

# Run PCoA function and show plot
trace.liens(Diss.mat)

# Calculate the R^2-like value (Legendre & Legendre 1998 p. 437... see also emails from PL below) 
# Extract eigenvalues
final$eig

# Calculate r^2-like value, which estimates how much variance in the dissimilarity matrix the first two axes of the PCoA explain.
# This is calculated as: sum(first two eigenvalues)/sum(all eigenvalues)
sum(final$eig[1:2])/sum(final$eig) # 0.2721413